// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.util.*;
import java.io.*;


public class PropertyManager{
    
    public final static String propfilename="client.properties";
    public final static String IPA="ip";
    public final static String PRT="port";
    public final static String IFC="interface";
    public final static String FCS="fcs";
    public final static String HDR="header";
    public final static String SUB="subnet";
    public final static String SCH="search";
    public final static String RET="retries";
    public final static String TMO="timeout";
    public final static String LNG="language";
    private final static String propathname="./";

    public final static List<String>
            CLICNFKEYS=Arrays.asList(new String[]{IPA, PRT, IFC,
                                                  FCS, HDR, SUB,
                                                  SCH, RET, TMO,
                                                  LNG});
    
    public final static List<String>
            HDRVALUES=Arrays.asList(
                        new String[]{"AUTO","802.11","RADIOTAP"});

    public final static List<String>
            SCHVALUES=Arrays.asList(new String[]{"COMP", "PART"});

    private static boolean validate(String key, String val){
        if(val==null)
            return false;
        else if(key.equals(IFC))
            return true;
        else if(key.equals(PRT)){
            try{
                int tmp=Integer.parseInt(val);
                return tmp>1023 && tmp<65536;
            }catch(NumberFormatException nfe){}
            return false;
        }else if(key.equals(IPA)){
            String[] ipfrags=val.split("\\.");
            if(ipfrags.length==4)
                try{
                    boolean ok=true;
                    int tmp=0;
                    for(String part: ipfrags)
                        tmp=Integer.parseInt(part);
                        ok=ok && tmp>=0 && tmp<256;
                        
                     return ok;
                }catch(NumberFormatException nfe){}
            return false;
        }else if(key.equals(FCS))
            return (val.equals("y") || val.equals("n"));
        else if(key.equals(HDR))
            return HDRVALUES.contains(val.toUpperCase());
        else if(key.equals(SUB)){
            if(val.indexOf(".")>0)
                try{
                    int index=val.indexOf(".");
                    int len=val.length();
                    int fb=Integer.parseInt(val.substring(0, index));
                    int sb=Integer.parseInt(val.substring(index+1, len));
                    boolean ok=false;
                    ok=(fb==10) || (sb>-1 && sb<256);
                    ok=ok || (fb==172 && sb>15 && sb<32);
                    ok=ok || (fb==192 && sb==168);
                    ok=ok || (fb==169 && sb==254);
                    return ok;
                }catch(NumberFormatException ne){
                    return false;
                }
            else
                return false;
        }else if(key.equals(SCH))
            return SCHVALUES.contains(val.toUpperCase());
        else if(key.equals(RET))
            try{
                int ret=Integer.parseInt(val);
                return ret>=Limits.MINRETS && ret<=Limits.MAXRETS;
            }catch(NumberFormatException ne){
                return false;
            }
        else if(key.equals(TMO))
            try{
                int tmout=Integer.parseInt(val);
                return tmout>=Limits.MINTIME && tmout<=Limits.MAXTIME;
            }catch(NumberFormatException ne){
                return false;
            }
        else if(key.equals(LNG)){
            String lang=val.toUpperCase();
            return lang.equals("SPANISH") || lang.equals("ENGLISH");
        }
        else
            return false;
    }

    public static Properties loadProperties(){
        Properties props=new Properties();        
        
        try{
            File f=new File(propathname+propfilename);
            InputStream is=null;
            Reader rd=null;

            if(f.canRead())
                rd=new FileReader(f);
            else{
                is=PropertyManager.class.getResourceAsStream(propfilename);
                if(is!=null)
                    rd=new InputStreamReader(is,"UTF-8");
            }
                
            if(rd!=null){
                props.load(rd);
                Iterator it=props.keySet().iterator();
                String key=null;
                boolean ok=false;

                while(it.hasNext()){
                    key=(String)it.next();
                    ok=validate(key, props.getProperty(key));
                    if(ok)
                        continue;
                    else
                        it.remove();
                }
                rd.close();
                return props;
            }else
                return null;
            
        }catch(IOException io){
            System.err.println("Error while recovering configuration: ");
            io.printStackTrace(System.err);
            return null;
        }            
    }

    public static boolean saveProperties(Properties props){
        try{
            File f=new File(propathname+propfilename);
            if(!f.exists() || f.canWrite()){
                Writer wr=new FileWriter(f);
                props.store(wr, "Client Setup");
                wr.close();
                return true;
            }
            else
                throw new IOException();            
            
        }catch(IOException io){
            System.err.println("Configuration not saved");
            io.printStackTrace(System.err);
            return false;
        }
    }
}
